# awl.tests.test_commands.py
import os, mock

from django.contrib.auth.models import User
from django.core.management import call_command
from django.test import TestCase

# ============================================================================

class CommandTests(TestCase):
    def test_create_admin(self):
        call_command('create_test_admin')

        # verify user created by script exists
        User.objects.get(username='admin')

    def test_run_script(self):
        filename = os.path.abspath(os.path.join(
            os.path.dirname(__file__), 'data/runme.py'))

        call_command('run_script', filename)

        # verify user created by script exists
        User.objects.get(username='fakeuser')

    @mock.patch('os.remove')
    def test_wipe_migrations(self, mocked_remove):
        call_command('wipe_migrations')

        self.assertTrue(mocked_remove.called)



# awl.managment.commands.wipe_migrations.py
#
# searches through the INSTALLED_APP listing looking for any apps that are in
# the same base directory structure as the command and removes any migration
# scripts belonging to them
import os

from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand

class Command(BaseCommand):
    def handle(self, *args, **options):
        project_dir = settings.BASE_DIR
        search_dir = os.path.abspath(os.path.dirname(project_dir))

        for app in apps.get_apps():
            filename = app.__file__
            if filename.startswith(search_dir):
                app_dir = os.path.abspath(os.path.dirname(filename))
                migrations = os.path.join(app_dir, 'migrations')
                if os.path.isdir(migrations):
                    for filename in os.listdir(migrations):
                        if filename.startswith('_'):
                            continue

                        os.remove(os.path.join(migrations, filename))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Counter',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=30)),
                ('value', models.BigIntegerField(default=0)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Lock',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=30)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# awl.context_processors.py
import os

def extra_context(request):
    """Adds useful global items to the context for use in templates.

    * *request*: the request object
    * *HOST*: host name of server
    * *IN_ADMIN*: True if you are in the django admin area
    """
    host = os.environ.get('DJANGO_LIVE_TEST_SERVER_ADDRESS', None) \
        or request.get_host()
    d = {
        'request':request,
        'HOST':host,
        'IN_ADMIN':request.path.startswith('/admin/'),
    }

    return d



# awl.templatetags.awltags.py

from django import template

register = template.Library()

# ============================================================================

@register.filter
def getitem(dictionary, keyvar):
    """Custom django template filter that allows access to an item of a
    dictionary through the key contained in a template variable.  Example:

    .. code-block:: python

        context_data = {
            'data':{
                'foo':'bar',
            },
            'key':'foo',
        }

        template = Template('{% load awltags %}{{data|getitem:key}}')
        context = Context(context_data)
        result = template.render(context)

        >>> result
        'bar'
    """
    return dictionary[keyvar]


@register.tag
def accessor(parser, token):
    """This template tag is used to do complex nested attribute accessing of
    an object.  The first parameter is the object being accessed, subsequent
    paramters are one of: 

    * a variable in the template context
    * a literal in the template context
    * either of the above surrounded in square brackets

    For each variable or literal parameter given a `getattr` is called on the
    object, chaining to the next parameter.  For any sqaure bracket enclosed
    items the access is done through a dictionary lookup.

    Example::

        {% accessor car where 'front_seat' [position] ['fabric'] %}

    The above would result in the following chain of commands:

    .. code-block:: python

        ref = getattr(car, where)
        ref = getattr(ref, 'front_seat')
        ref = ref[position]
        return ref['fabric']

    This tag also supports "as" syntax, putting the results into a template
    variable::

        {% accessor car 'interior' as foo %}
    """
    contents = token.split_contents()
    tag = contents[0]
    if len(contents) < 3:
        raise template.TemplateSyntaxError(('%s requires at least two '
            'arguments: object and one or more getattr parms') % tag)

    as_var = None
    if len(contents) >= 4:
        # check for "as" syntax
        if contents[-2] == 'as':
            as_var = contents[-1]
            contents = contents[:-2]

    return AccessorNode(contents[1], contents[2:], as_var)


class AccessorNode(template.Node):
    def __init__(self, obj_name, parms, as_var):
        self.obj_name = obj_name
        self.parms = parms
        self.as_var = as_var

    def render(self, context):
        ref = context[self.obj_name]
        for parm in self.parms:
            if parm[0] == '"' or parm[0] == "'":
                # parm is a literal
                ref = getattr(ref, parm[1:-1])
            elif parm[0] == '[':
                # parm is a dictionary lookup
                if parm[1] == '"' or parm[1] == "'":
                    # dict key is a literal
                    ref = ref[parm[2:-2]]
                else:
                    # dict key is a template var
                    key = context[parm[1:-1]]
                    ref = ref[key]
            else:
                # parm is a template var
                attr = context[parm]
                ref = getattr(ref, attr)

        if self.as_var:
            context[self.as_var] = ref
            return ''

        return ref
